/* Copyright 1997 Lucent Technologies Inc. - All Rights Reserved */

#include <unistd.h>
#include <stdio.h>
#include <errno.h>
#include <sys/param.h>
#include <sys/file.h>
#include <fcntl.h>
#include <sys/time.h>
#include <ctype.h>
#include <sys/stat.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <signal.h>

#include "get.h"
#include "client.h"
#include "crypto.h"
#include "arg.h"
#include "util.h"

int srvfd = -1;
int debug = 0;

static void
timeout(void) {
	fprintf(stderr, "timeout\n");
	exit(1);
}

void
expecthear(int fd, char *expect) {
	char buf[512];
	int n;

	if (debug > 1)
		log("expecting: %s", expect);

	n = cread(fd, buf, sizeof(buf));
	if (n < 0) {
		cperror("read server response");
		exit(11);
	}
	if (strcmp(buf, expect)) {
		fprintf(stderr, "%s\n",buf);
		exit(11);
	}
}


int
usage(void) {
	fprintf(stderr,
		"usage: cput [-c cservfn] [-d] [-s] [-k keyfile] [-p port] [-h host] service locfn [remfn]\n");
	return(1);
}

int
main(int argc, char *argv[]) {
	struct stat sbuf;
	char buf[MAXIO];
	char *hostname = 0;
	char *service = 0;
	KEY key;
	char *csrvfile = CSRVFILE;
	int serverport = SERVERPORT;
	char *putfile = 0;
	char *outfile = 0;
	char *keyfile = 0;
	char *host = 0;
	u_long t0, t1;
	char *ep, *er;
	int sflg = 0;
	int bytes = 0;
	int infd;
	int x;

	ARGBEGIN{
	case 'c': csrvfile = ARGF();		break;
	case 'd': debug++;			break;
	case 'h': hostname = ARGF();		break;
	case 'k': keyfile = ARGF();		break;
	case 'p': serverport = atoi(ARGF());	break;
	case 's': sflg++;			break;
	default:
		exit(usage());
	}ARGEND;

	if (argc != 2 && argc != 3)
		exit(usage());

	service = *argv++;
	putfile = *argv++;
	if (argc == 3)
		outfile = *argv;
	else
		outfile = putfile;
	host = get_service_host(csrvfile, service);
	if (hostname == 0)
		hostname = host;
	get_key(keyfile, service, key);

	if(stat(putfile,&sbuf)) {
		perror(putfile);
		exit(6);
	}

	if (S_ISDIR(sbuf.st_mode)) {
		fprintf(stderr, "cput: can't put a directory: %s\n", putfile);
		exit(1);
	}

	if ((srvfd = tcpconnect(hostname, serverport)) < 0) {
		fprintf(stderr, "cput: connection to %s:%d: %s\n",
			hostname, serverport, strerror(errno));
		exit(2);
	}

	ep = start_client_crypto(srvfd, key, service);
	if (ep) {
		fprintf(stderr, "cput: %s, %s: %s\n", hostname, service, ep);
		exit(3);
	}
	if (debug)
		log("crypto setup complete");

	/* send the desired filename */
	sprintf(buf, "put %o %d %d %s",
		(unsigned short)sbuf.st_mode, sbuf.st_mtime,
		(u_long)sbuf.st_size, outfile);
	if (debug > 1)
		log("writing: %s", buf);
	signal(SIGALRM, (void *)timeout);
	alarm(20);

	cwrites(srvfd, buf);
	expecthear(srvfd, "ok");

	if((infd = open(putfile,O_RDONLY)) < 0) {
		perror(putfile);
		close(srvfd);
		exit(7);
	}

	t0 = rtime();
	if (debug)
		log("sending file, %d bytes", sbuf.st_size);

	alarm(0);
	er = writenetfile(infd, srvfd, sbuf.st_size);
	if (er) {
		fprintf(stderr, "write error: %s\n", er);
		exit(1);
	}

	t1 = rtime();
	if(sflg) {
		double t = t1 - t0;

		fprintf(stderr, "cput: %s: %d bytes, %.2f secs, %.2f KB/s\n",
			putfile, sbuf.st_size, t/1000.0, sbuf.st_size/1024/(t/1000.0));
	}
	close(infd);

	expecthear(srvfd, "bye");
	exit(0);
}
